<?php 
/*** COPYRIGHT NOTICE *********************************************************
 *
 * Copyright 2009-2017 ProjeQtOr - Pascal BERNARD - support@projeqtor.org
 * Contributors : Eliott LEGRAND (from Salto Consulting - 2018) 
 *
 * This file is part of ProjeQtOr.
 * 
 * ProjeQtOr is free software: you can redistribute it and/or modify it under 
 * the terms of the GNU Affero General Public License as published by the Free 
 * Software Foundation, either version 3 of the License, or (at your option) 
 * any later version.
 * 
 * ProjeQtOr is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
 * FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for 
 * more details.
 *
 * You should have received a copy of the GNU Affero General Public License along with
 * ProjeQtOr. If not, see <http://www.gnu.org/licenses/>.
 *
 * You can get complete code of ProjeQtOr, other resource, help and information
 * about contributors at http://www.projeqtor.org 
 *     
 *** DO NOT REMOVE THIS NOTICE ************************************************/
// ELIOTT - LEAVE SYSTEM
/** ============================================================================
 * LeaveMain
 * Management of the leave
 ** ============================================================================ */  
// RULES :
// x. At creation :
//      - Initialize :
//          idUser with the user connected id
//              done in save
//          idStatus to 1 (first status in all workflow)
//              done in save and construct
//          requestDateTime to the current date time
//              done in save
//          nbDays to 1
//              Done in construct
//          startDate and endDate at the first valid date ie :
//              More near of current date and not included in existing leave or off day
//              Done in getValidationScript
//                 Not in construct BECAUSE :
//                  A bug arising in construct if a new Leave() is called or a getSqlElementsFromCriteria()
//          startAMPM is checked to AM
//          endAMPM is checked to PM
//              Done in drawStartAMPM and drawEndAMPM
// x. idEmployee :
//      List of visible employee by the connected user
//          done throw its name (see objectDetail.php : call getUserVisibleResourcesList(true, "List",'', false, true)
//      Readonly if connected user is'nt Manager
//          done is construct
//      Required if connected user is Manager
//          done is construct
//      At creation, initialized with connected user
//          done in construct
// x. EndDate >= StartDate
//      Done in control and getValidationScript
// x. Manager (or actual delegation)
//      Can create leaves for employees that he manage
//          TO DO
//      Can update, delete leaves that he has created
//          TO DO
//      Can only update status leaves for employees that he manage if he is not the creator
//          TO DO
// x. Notification
//      If Notification system is activ => At creation or update of a leave, create a notification for the manager
//          TO DO
//      If leave's status change, create a notification for the Employee
//          TO DO
// x. Email
//      If Notification system is activ => At creation or update of a leave, create a notification for the manager
//          TO DO
//      If leave's status change, send an email to the Employee
//          TO DO    
// ============================================================================

require_once('_securityCheck.php'); 
class LeaveMain extends SqlElement {

  // List of fields that will be exposed in general user interface
  public $_sec_description;
    public $id;    // redefine $id to specify its visible place
    public $idUser; // Creator of the leave
    public $idLeaveType;
    public $startDate;
    public $_spe_startAMPM;
    public $startAMPM;
    public $endDate;
    public $_spe_endAMPM;
    public $endAMPM;
    public $nbDays;
    public $_spe_nbDaysRemaining;
    public $_nbRemain;
    public $comment;
  public $_sec_treatment;
    public $_spe_isJustifiable;
    public $idStatus;
    public $idEmployee; // Employee that takes the leave
    public $requestDateTime;
    public $idResource; //the id of the resource who validated the leave
    public $processingDateTime;
    public $idle;
  public $_Attachment=array();

  public $_nbColMax=3;
  
    // Define the layout that will be used for lists
    private static $_layout='
        <th field="id" formatter="numericFormatter" width="4%" ># ${id}</th>
        <th field="nameEmployee" formatter="thumbName22" width="12%" >${employee}</th>
        <th field="nameLeaveType" width="5%" >${type}</th>
        <th field="startDate" width="8%" formatter="dateFormatter">${startDate}</th>
        <th field="endDate" width="8%" formatter="dateFormatter">${endDate}</th>
        <th field="nbDays" formatter="numericFormatter" width="4%" ># ${nbDays}</th>
        <th field="colorNameStatus" width="8%" formatter="colorNameFormatter">${idStatus}</th>
        <th field="idle" width="5%" formatter="booleanFormatter">${idle}</th>
        ';

    private static $_fieldsAttributes=array("startDate" => "required",
        "endDate" => "required",
        "idLeaveType" => "required",
        "nbDays" => "readonly",
        "idResource" => "readonly,nocombo,invisible",
        "processingDateTime" => "readonly,invisible",
        "idEmployee" => "readonly",
        "requestDateTime" => "readonly",
        "startAMPM" => "hidden",
        "endAMPM" => "hidden",
        "idStatus" => "required",
        "idUser" => "readonly",
        "idle" => "readonly"
        );  

    private $___dFieldsAttributes=array();

  
    private static $_colCaptionTransposition = array('idUser' => 'issuer',
                                                    'idResource' => 'validator'
                                                   );

    private static $_databaseColumnName = array();

    private static $_databaseTableName = 'employeeleaveperiod'; 
   /** ==========================================================================
   * Constructor
   * @param $id the id of the object in the database (null if not stored yet)
   * @return void
   */ 
  function __construct($id = NULL, $withoutDependentObjects=false) {
        
    parent::__construct($id,$withoutDependentObjects);
    if($this->idStatus!=NULL and $this->idStatus!=1){
        self::$_fieldsAttributes['idLeaveType']='readonly';
        self::$_fieldsAttributes['startDate']='readonly';
        self::$_fieldsAttributes['endDate']='readonly';
        self::$_fieldsAttributes['comment']='readonly';
        self::$_fieldsAttributes['idStatus']='readonly';
        $this->___dFieldsAttributes["processingDateTime"] = "readonly";
        $this->___dFieldsAttributes["idResource"] = "readonly,nocombo";
    }
    //At the creation of a new leave 
    if($id==NULL){
        $this->___dFieldsAttributes["requestDateTime"] = "invisible";
        // Initialize idEmployee with the user connected id
        $this->idEmployee = getSessionUser()->id;
        //Initialize idStatus to 1 (first status in all workflow)
        $this->idStatus=1;
        //Initialize nbDays to 1
        $this->nbDays=1;
        
        $lvType= new LeaveType();
        $lvTypeList = $lvType->getSqlElementsFromCriteria(null,false,"idle=0","id DESC");
        if (count($lvTypeList)==0) {
            $this->idLeaveType= null;
        } else {
            $this->idLeaveType = $lvTypeList[0]->id;
        }
    }
    // If connected user is'nt Manager and is'nt leave Admin and is'nt Manager of Employee
    if (!isLeavesManager(getSessionUser()->id) and 
        !isLeavesAdmin(getSessionUser()->id) and 
        !isManagerOfEmployee(getSessionUser()->id, $this->idEmployee)) {
        // idEmployee and idStatus are Readonly
        self::$_fieldsAttributes['idEmployee']="readonly";
        $this->___dFieldsAttributes["idEmployee"] = "readonly";
        self::$_fieldsAttributes['idStatus']="readonly";        
    } else {
        // If Status != Recorded (idStatus !=1)
        if ($this->idStatus !=1) {
            // idEmployee is readonly
            self::$_fieldsAttributes['idEmployee']="readonly";
            $this->___dFieldsAttributes["idEmployee"] = "readonly";
            // If connected user is manager or leave admin
            if (isLeavesManager(getSessionUser()->id) or 
                isLeavesAdmin(getSessionUser()->id) or 
                isManagerOfEmployee(getSessionUser()->id, $this->idEmployee) ) {
                // idStatus = required
                self::$_fieldsAttributes['idStatus']="required";
            } else {
                self::$_fieldsAttributes['idStatus']="readonly";                        
            }
        } else { // idStatus =1 AND (manager or leave admin)
            if ($this->id==null) {
                // idEmployee is required
                self::$_fieldsAttributes['idEmployee']="required";
                $this->___dFieldsAttributes["idEmployee"] = "required";
            } else {
                if ($this->idUser != getSessionUser()->id) { // Is'nt the creator
                    // idEmployee, idLeaveType, startDate, endDate, startAMPM, endAMPM are readonly
                    self::$_fieldsAttributes['idEmployee']="readonly";
                    $this->___dFieldsAttributes["idEmployee"] = "readonly";
                    $this->___dFieldsAttributes["idLeaveType"] = "readonly";
                    $this->___dFieldsAttributes["startDate"] = "readonly";
                    $this->___dFieldsAttributes["endDate"] = "readonly";
                    $this->___dFieldsAttributes["startAMPM"] = "readonly,hidden";
                    $this->___dFieldsAttributes["endAMPM"] = "readonly,hidden";                    
                }
            }
        }    
    }
    
    // idUser is'nt readonly if connected user is admin
    if (getSessionUser()->idProfile==1) {
        self::$_fieldsAttributes['idUser']="";
    }
  }

   /** ==========================================================================
   * Destructor
   * @return void
   */ 
  function __destruct() {
    parent::__destruct();
   }


// ============================================================================**********
// GET STATIC DATA FUNCTIONS
// ============================================================================**********
  
  /** ==========================================================================
   * Return the specific layout
   * @return the layout
   */
  protected function getStaticLayout() {
    return self::$_layout;
  }
  
  /** ==========================================================================
   * Return the specific fieldsAttributes
   * @return the fieldsAttributes
   */
  protected function getStaticFieldsAttributes() {
    return self::$_fieldsAttributes;
  }
  
  /** ============================================================================
   * Return the specific colCaptionTransposition
   * @return the colCaptionTransposition
   */
  protected function getStaticColCaptionTransposition($fld=null) {
    return self::$_colCaptionTransposition;
  }

  /** ========================================================================
   * Return the specific databaseTableName
   * @return the databaseTableName
   */
  protected function getStaticDatabaseColumnName() {
    return self::$_databaseColumnName;
  }
    
  /** ========================================================================
   * Return the specific databaseTableName
   * @return the databaseTableName
   */
  protected function getStaticDatabaseTableName() {
    $paramDbPrefix=Parameter::getGlobalParameter('paramDbPrefix');
    return $paramDbPrefix . self::$_databaseTableName;
  }
  
  /**
   * Get the dynamic attributes (or static if dynamic not found) of the field that name is passed in parameter
   * @param String $fieldName : The fieldName for witch get attributes
   * @return String Attributes of the field
   */
  public function getFieldAttributes($fieldName) {
    if (array_key_exists ( $fieldName, $this->___dFieldsAttributes )) {
      return $this->___dFieldsAttributes[$fieldName];
    } else {
        return parent::getFieldAttributes($fieldName);
    }      
  }
  
 /* ========================================================================================
  * SQL ELEMENT FUNCTIONS
    ======================================================================================== */   
  
  /** =========================================================================
   * control data corresponding to Model constraints
   * @param void
   * @return "OK" if controls are good or an error message 
   *  must be redefined in the inherited class
   */
  public function control(){
    $result="";
    
        // Can't take leave if employee has no unclosed contract
        $empl = new Employee($this->idEmployee);
        if ($empl->hasAllClosedContracts()) {
            $result.= "<br/>".i18("NoUnclosedContractForThisEmployeeCantTakeLeaves");
            return $result;
        }
    
        //test if a leaveType exists
        $lvType = new LeaveType();
        $lvTypeRes = $lvType->getSqlElementsFromCriteria(array());
        if(! $lvTypeRes){
            $result.='<br/>' . i18n('leaveTypesDontExist');
        }
        
        // leaveType can't be null
        if ($this->idLeaveType==null) {
            $result.= "<br/>".i18("LeaveTypeMandatory");
        }
        
        // Can't take a leave after end of contract
        $contract = EmploymentContract::getActualEmploymentContractForAnEmployee($this->idEmployee);
        if ($contract->endDate!=null) {
            $contractEndDate = $contract->endDate->format("Y-m-d");
            $endDateString = $this->endDate->format("Y-m-d");
            if ($endDateString > $contractEndDate) {
                $result.='<br/>' . i18n('CantTakeLeaveAfterEndOfContract');            
            }
        }
        
        if ($this->id!=null) {
            $old=$this->getOld();
            //Only him self or it's manager or leave Admin can change something if status is 1 (Recorded)
            if ($this->idStatus==1) {
                if ($this->idEmployee != getSessionUser()->id and 
                    !isManagerOfEmployee(getSessionUser()->id, $this->idEmployee) and
                    !isLeavesAdmin(getSessionUser()->id)) {
                    if ($old->idUser != $this->idUser or 
                        $this->idLeaveType != $old->idLeaveType or
                        $this->idEmployee != $old->idEmployee or
                        $this->idResource != $old->idResource or
                        $this->idle != $old->idle or
                        $this->endDate != $old->endDate or
                        $this->startDate != $old->startDate or
                        $this->startAMPM != $old->startAMPM or
                        $this->endAMPM != $old->endAMPM or
                        $this->comment != $old->comment) {
                        $result .= "<br/>".i18n("OnlySelfEmployeeOrManagerCanChangeSomething");
                    }
                }
            }else{
                if (!isManagerOfEmployee(getSessionUser()->id, $this->idEmployee) &&
                    !isLeavesAdmin(getSessionUser()->id)) {
                    $result .= "<br/>".i18n("EmployeeManagerOrLeaveSystemAdminCanChangeStatus");
                }
            }
        }

        //if nbDays==0, then the leave is composed entirely of OffDays or the dates are wrong
        if($this->nbDays<=0){
            $result.='<br/>' . i18n('nbDaysCantBeInferiorOrEqualToZero');
        }
        
        //if the endDate is set before the startDate
        if($this->startDate > $this->endDate){
            $result.='<br/>' . i18n('invalidEndDate');
        }
        if ($contract!=null) {
            $contractType = $contract->getEmploymentContractType();
            if ($contractType->id>0) {
                $crit = array("idle" => '0',
                              "idEmploymentContractType" => $contractType->id,
                              "idLeaveType" => $this->idLeaveType
                             );
                $lvTpOf = SqlElement::getFirstSqlElementFromCriteria("LeaveTypeOfEmploymentContractType", $crit);
                if ($lvTpOf) {
                    if ($lvTpOf->nbDaysAfterNowLeaveDemandIsAllowed!=null) {
                        $intervalPrior = $lvTpOf->nbDaysAfterNowLeaveDemandIsAllowed;                        
                    } else {
                        $intervalPrior = 9999;
                    }
                    if ($lvTpOf->nbDaysBeforeNowLeaveDemandIsAllowed!=null) {
                        $intervalFutur = $lvTpOf->nbDaysBeforeNowLeaveDemandIsAllowed;
                    } else {
                        $intervalFutur = 9999;
                    }
                } else {
                    $intervalPrior = 9999;
                    $intervalFutur = 9999;                    
                }
            } else {
                $intervalPrior = 9999;
                $intervalFutur = 9999;
            }
        } else {
            $intervalPrior = 9999;
            $intervalFutur = 9999;
        }
        if ($intervalPrior<9999) {
            if( $this->startDate < ((new DateTime('now'))->sub(new DateInterval('P'.$intervalPrior.'D'))->format('Y-m-d')) and $this->idStatus==1 ){
                $result.='<br/>' . i18n('cantTakeAStartDatePriorTo').' '. $intervalPrior.' '. i18n('days'); 
            }
        }
        if ($intervalFutur<9999) {
            //if the startDate is set after one year in advance, MUST BE MODIFIED IN THE FUTURE to be able to choose the date interval allowed to take a leave in the future (example: one year in the futur) 
            if( $this->startDate > ((new DateTime('now'))->add(new DateInterval('P'.$intervalFutur.'Y'))->format('Y-m-d')) and $this->idStatus==1){
                $result.='<br/>' . i18n('cantTakeAStartDateSuperiorTo'). ' '. $intervalFutur.' '.i18n('days');
            }
        }
                
        //if startDate and endDate are set to the same day and the leave start the afternoon and end in the morning, throw an error
        if($this->startDate==$this->endDate and $this->startAMPM==='PM' and $this->endAMPM==='AM'){
            $result.='<br/>' . i18n('CantStartPMEndAMIfOneDay');
        }
        

    //to test if the leave overlap with other existing leave 
        $thisStartDateRqFormat = (new DateTime($this->startDate))->format('Ymd');
        $thisEndDateRqFormat = (new DateTime($this->endDate))->format('Ymd');  
        //the request select all the leaves of the requester that overlap with the current one which are not cancelled (idStatus <> 9)
        //for the overlap, the half-days are taken into account
        $clauseWhere="idEmployee=".$this->idEmployee." AND idStatus <> 9 "//select the leaves of the requester which are not cancelled
                //and the leaves included between the dates of the new leave
                . "AND ((startDate > ".$thisStartDateRqFormat." AND endDate < ".$thisEndDateRqFormat.") "
                //and the leaves which include the dates of the new leaves
                . "OR (startDate < ".$thisStartDateRqFormat." AND endDate > ".$thisEndDateRqFormat.") "
                //and the leaves which include the startDate of the new leave (and their endDate must be inferior or equal to the endDate of the new leave so the request doesn't include all the leaves after endDate
                . "OR (startDate <= ".$thisStartDateRqFormat." AND endDate > ".$thisStartDateRqFormat."  AND endDate <= ".$thisEndDateRqFormat.") "
                //and the leaves which include the endDate of the new leave 
                . "OR (startDate < ".$thisEndDateRqFormat." AND endDate >= ".$thisEndDateRqFormat." AND startDate >= ".$thisStartDateRqFormat.") "
                //and the leaves which does not respect a particular case (example: it's possible to take a leave which start the 14/06/18 PM and end the 16/06/18 AM with two leaves with the first ending the 14/06/18 AM and the second beginning the 16/06/18 PM)
                . "OR (endDate=".$thisStartDateRqFormat." AND startDate=".$thisEndDateRqFormat." AND (NOT (endAMPM='AM' AND '".$this->startAMPM."'='PM')) AND (NOT ('".$this->endAMPM."'='AM' AND startAMPM='PM')) AND (NOT(".$thisStartDateRqFormat."=".$thisEndDateRqFormat.")) ) "
                //...
                . "OR (endDate=".$thisStartDateRqFormat." AND NOT (endAMPM='AM' AND '".$this->startAMPM."'='PM') AND (NOT(".$thisStartDateRqFormat."=".$thisEndDateRqFormat.")) ) "
                
                . "OR (startDate=".$thisEndDateRqFormat." AND NOT ('".$this->endAMPM."'='AM' AND startAMPM='PM') AND (NOT(".$thisStartDateRqFormat."=".$thisEndDateRqFormat.")) )  "
                
                . "OR (startDate=".$thisStartDateRqFormat." AND startDate=".$thisEndDateRqFormat." AND startDate=endDate AND ( NOT( (startAMPM='PM' AND endAMPM='PM' AND '".$this->startAMPM."'='AM' AND '".$this->endAMPM."'='AM') OR (startAMPM='AM' AND endAMPM='AM' AND '".$this->startAMPM."'='PM' AND '".$this->endAMPM."'='PM') ) ) ) "
                . ")"; 
         if($this->id != NULL){//to exclude this leave of the request
             $clauseWhere.="AND id <> ".$this->id;
         }
        $clauseOrderBy = "startDate, endDate ASC";
        $queryResult=$this->getSqlElementsFromCriteria ( null, false, $clauseWhere, $clauseOrderBy );
        if($queryResult){//if the query returned some leaves then error
            $result.='<br/>' . i18n('LeavesCannotOverlapWithEachOther');
        }       
    //end overlap test
   	
        $defaultControl=parent::control();
   	if ($defaultControl!='OK') {
   		$result.=$defaultControl;
   	}
   	
   	if ($result=="") $result='OK';
	
    return $result;
}

  
  /**=========================================================================
   * Overrides SqlElement::save() function to add specific treatments
   * @see persistence/SqlElement#save()
   * @return the return message of persistence/SqlElement#save() method
   */
  public function save() {
    $old=$this->getOld();
      //ajout eliott 
    
    // Initialize idUser with the user connected id
    $this->idUser = getSessionUser()->id;

    if ($this->id==NULL){
        // Initialize idEmployee with the user connected id if is null
        if ($this->idEmployee==null) {
            $this->idEmployee = getSessionUser()->id;
        }    
        // Initialize requestDateTime to current date time
        $this->requestDateTime = (new DateTime('now'))->format('Y-m-d\TH:i:s');
        // Initialize idStatus to 1 (first status in all workflow)
        $this->idStatus = 1;
    }
    
    //to save the id of the validator
    if($this->idStatus!=NULL and $this->idStatus!=1){
        $this->idResource = getSessionUser()->id;
        $this->processingDateTime = (new DateTime('now'))->format('Y-m-d\TH:i:s');   
    } elseif($this->idStatus==NULL or $this->idStatus==1){//if the leave is set again to recorded or his status hasn't been changed yet    
        $this->idResource = NULL;
        $this->processingDateTime = NULL;
    } 
   //fin ajout eliott
   
    //to set the hidden parameters startAMPM and endAMPM 
    if (RequestHandler::getValue('startAM')=='on'){
        $this->startAMPM='AM';
    }
    if (RequestHandler::getValue('startPM')=='on'){
        $this->startAMPM='PM';
    }
    if (RequestHandler::getValue('endAM')=='on'){
        $this->endAMPM='AM';
    }
    if (RequestHandler::getValue('endPM')=='on'){
        $this->endAMPM='PM';
    }
    
    $old=$this->getOld();
    //to recalculate nbDays 
    if($old->startDate!=$this->startDate || $old->endDate!=$this->endDate || $old->startAMPM!=$this->startAMPM || $old->endAMPM!=$this->endAMPM || $old->nbDays!=$this->nbDays){
        $this->calculateNbDays();
    }
    
    //to update the attribute leftQuantity of the leaveEarned(s) concerned by this leave 
    $resultClass="";
    $id=$this->id;
    if($this->id==null){//create
        $resultClass.=$this->updateLeftQOfLeaveEarned($this->idLeaveType,$this->idEmployee,$this->nbDays,null,true);
    }else if($this->idEmployee != $old->idEmployee || $this->idLeaveType!=$old->idLeaveType){//if the leaveType/idEmployee changed, then delete the leave and create a new (simpliest way to deal with these changes)
        $resultClass.=$this->updateLeftQOfLeaveEarned($old->idLeaveType,$old->idEmployee,$old->nbDays,null,false,false,true);
        $resultClass.=$this->updateLeftQOfLeaveEarned($this->idLeaveType,$this->idEmployee,$this->nbDays,null,true);
    }else if ($old->nbDays != $this->nbDays){//update
        $resultClass.=$this->updateLeftQOfLeaveEarned($this->idLeaveType,$this->idEmployee,$this->nbDays,(float)$old->nbDays,false, true);
    }else if ($old->idStatus!=$this->idStatus && $this->idStatus == 9){//delete
        $resultClass.=$this->updateLeftQOfLeaveEarned($this->idLeaveType,$this->idEmployee,$this->nbDays,null,false, false, true);
    }else if ($old->idStatus!=$this->idStatus && $old->idStatus==9 && $this->idStatus == 1){//delete
        $resultClass.=$this->updateLeftQOfLeaveEarned($this->idLeaveType,$this->idEmployee,$this->nbDays,null,true);
    }
    //if there was an error during the execution of updateLeftQOfLeaveEarned(), stop
    if($resultClass!=""){
        return $resultClass;
    }
    
    $resultClass .= parent::save();

    if(strpos($resultClass,"OK")===false){
        return $resultClass;
    }
    
    //create the plannedWorks for the leave if the status is registered
    if ($this->idStatus==1) {
        $resultS = $this->createWorkOrPlannedWorkForLeave('PlannedWork');        
    } else { // delete plannedWorks
        $clause = "idLeave=$this->id";
        $pWork = new PlannedWork();
        $resultS = $pWork->purge($clause);
        if(strpos($resultS, "OK")===false){
            return $resultS;   
        }        
    }
    
    //create the works for the leave if the status changed from registered to validated
    if ( $old->idStatus == 1 && $this->idStatus == 12){
        $resultS = $this->createWorkOrPlannedWorkForLeave('Work');
    }
    
    //delete the works for the leave if the status changed from validated to recorded
    if ($this->idStatus==1 and $old->idStatus==12) {
        $clause = "idLeave=$this->id";
        $work = new Work();
        $resultS = $work->purge($clause);
        if(strpos($resultS, "OK")===false){
            return $resultS;   
        }
    }
    
    // Send Notification or email and alert
    if (getLastOperationStatus($resultClass)=='OK') {
        $this->sendLeaveInfo($old);
    }
    
    return $resultClass;
  }
  
  
  /**
   * =========================================================================
   * control data corresponding to Model constraints, before deleting an object
   *
   * @param
   *          void
   * @return "OK" if controls are good or an error message
   *         must be redefined in the inherited class
   */
  public function deleteControl() {
    $result = "";
    /*
    //if there is no LeaveEarned for this leave, error
    $empLEList = $this->getEmpLeaveEarnedListOfThisLeave();
    if(! $empLEList){
        $result.='<br/>' . i18n('ErrorThisEmployeeDoesntHaveLeaveEarned');
    }
    */
    $defaultDeleteControl=parent::deleteControl();
    if ($defaultDeleteControl!='OK') {
        $result.=$defaultDeleteControl;
    }
    
    if ($result == "") {
      $result = 'OK';
    }
    
    return $result;
  }
  
  
  
  public function delete() {
    $result = "";
    //to update the column left of employeeLeaveEarned
    $result .= $this->updateLeftQOfLeaveEarned($this->idLeaveType,$this->idEmployee,$this->nbDays, null, false, false, true);
    //if there was an error during the execution of updateLeftQOfLeaveEarned(), stop
    if($result!=""){
        return $result;
    }
    
    $result .= parent::delete();

    //to delete all the works associated to this leave
    $work=new Work();
    $resultP=$work->purge("idLeave = ".$this->id);
    if(strpos($resultP, "OK")===false){
        return $resultP;   
    }
    //to delete all the planned works associated to this leave
    $pWork=new PlannedWork();
    $resultP=$pWork->purge("idLeave = ".$this->id);
    if(strpos($resultP, "OK")===false){
        return $resultP;   
    }

    return $result;
  }

 /* ========================================================================================
  * VALIDATION SCRIPT
    ======================================================================================== */   
  
   /** ==========================================================================
   * Return the validation sript for some fields
   * @return the validation javascript (for dojo frameword)
   */
  public function getValidationScript($colName) {
    //to set startDate and endDate at the creation of a new leave (this is not made in the __construct() as a call to getSqlElementsFromCriteria 
    //or to new Leave() seems to create an infinite loop in that function
    if($this->id==NULL){
        $firstOpenDay=$this->getFirstOpenDateAtCreation();
        $this->startDate = $firstOpenDay;
        $this->endDate = $firstOpenDay;
    }  
    $colScript = parent::getValidationScript($colName);
    
    if($colName=="idStatus"){//stuck here, dojo always reset the style after the style.display = none
        $colScript.='<script type="dojo/connect" event="onChange" >';
        $colScript.='if(dijit.byId("idStatus").get("value")==" " || dijit.byId("idStatus").get("value")==1){'
            . 'require(["dojo/query","dojo/dom-style"], function(query, domStyle){'
                . 'domStyle.set(query(".idResourceClass")[0], "display","none");'
                . 'domStyle.set(query(".processingDateTimeClass")[0], "display","none");'
            . '});'
        . '}'
        . 'else{'
            . 'require(["dojo/query","dojo/dom-style"], function(query, domStyle){'
                . 'domStyle.set(query(".idResourceClass")[0], "display","table-row");'
                . 'domStyle.set(query(".processingDateTimeClass")[0], "display","table-row");'
            . '});'
        . '}';
        $colScript .='var idRes = dijit.byId("idEmployee").value;';
        $colScript .='var idUser = '. getSessionUser()->id.';';        
        $colScript .='var nbDays = '. $this->nbDays.';';
        $colScript .='var idLeaveType = '. $this->idLeaveType.';';
        $colScript .='var idStatus = '. $this->idStatus.';';
        $colScript .= 'calculateNbRemainingDays("fromLeaveMain", idRes, idUser, nbDays, idStatus, idLeaveType);';
        $colScript .= '</script>';
    }
    
    if($colName=="startDate"){
        $colScript .='<script type="dojo/connect" event="onChange" >';
        $colScript .='var idRes = dijit.byId("idEmployee").value;';
        $colScript .='var idUser = '. getSessionUser()->id.';';        
        $colScript .='var nbDays = '. $this->nbDays.';';
        $colScript .='var idLeaveType = '. $this->idLeaveType.';';
        $colScript .='var idStatus = '. $this->idStatus.';';
        $colScript .='if (this.value > dijit.byId("endDate").value) {';
        $colScript .='    dijit.byId("endDate").setValue(this.value);';
        $colScript .='}';
        $colScript .="calculateHalfDaysForLeave('startDate', 'endDate', 'startAM', 'startPM', 'endAM', 'endPM', 'nbDays', idRes, idUser);";
        $colScript .= 'calculateNbRemainingDays("fromLeaveMain", idRes, idUser, nbDays, idStatus, idLeaveType);';
        $colScript .= '</script>';
    }
    
    if($colName=="endDate"){
        $colScript .='<script type="dojo/connect" event="onChange" >';
        $colScript .='var idRes = dijit.byId("idEmployee").value;';
        $colScript .='var idUser = '. getSessionUser()->id.';';        
        $colScript .='var nbDays = '. $this->nbDays.';';
        $colScript .='var idLeaveType = '. $this->idLeaveType.';';
        $colScript .='var idStatus = '. $this->idStatus.';';
        $colScript .='if (this.value < dijit.byId("startDate").value) {';
        $colScript .='    dijit.byId("startDate").setValue(this.value);';
        $colScript .='}';
        $colScript .="calculateHalfDaysForLeave('startDate', 'endDate', 'startAM', 'startPM', 'endAM', 'endPM', 'nbDays', idRes, idUser);";
        $colScript .= 'calculateNbRemainingDays("fromLeaveMain", idRes, idUser, nbDays, idStatus, idLeaveType);';
        $colScript .= '</script>';
    } 

    if($colName=="idEmployee"){
        $colScript .='<script type="dojo/connect" event="onChange" >';
        $colScript .='var idEmployee = this.value;';
        $colScript .='var idUser = '. getSessionUser()->id.';';        
        $colScript .='var nbDays = '. $this->nbDays.';';
        $colScript .='var idLeaveType = '. $this->idLeaveType.';';
        $colScript .='var idStatus = '. $this->idStatus.';';
        $colScript .="calculateHalfDaysForLeave('startDate', 'endDate', 'startAM', 'startPM', 'endAM', 'endPM', 'nbDays', idEmployee, idUser);";
        $colScript .="getLeftByLeaveType(idEmployee);";
        $colScript .= 'calculateNbRemainingDays("fromLeaveMain", idEmployee, idUser, nbDays, idStatus, idLeaveType);';
        $colScript .= '</script>';
    } 

    if($colName=="idLeaveType"){
        $colScript .='<script type="dojo/connect" event="onChange" >';
        $colScript .='var idRes = dijit.byId("idEmployee").value;';
        $colScript .='var idUser = '. getSessionUser()->id.';';
        $colScript .='var nbDays = '. $this->nbDays.';';
        $colScript .='var idLeaveType = '. $this->idLeaveType.';';
        $colScript .='var idStatus = '. $this->idStatus.';';
        $colScript .="calculateHalfDaysForLeave('startDate', 'endDate', 'startAM', 'startPM', 'endAM', 'endPM', 'nbDays', idRes, idUser);";
        $colScript .= 'calculateNbRemainingDays("fromLeaveMain", idRes, idUser, nbDays, idStatus, idLeaveType);';
        $colScript .= '</script>';
    } 

    return $colScript;
  }

 /* ========================================================================================
  * DRAW FUNCTIONS
    ======================================================================================== */   
  
  /** =========================================================================
   * Draw a specific item for the current class.
   * @param $item the item. Correct values are : 
   * @return an html string able to display a specific item
   *  must be redefined in the inherited class
   */
  public function drawSpecificItem($item,$readOnly=false,$refresh=false){
    $result="";
    
    switch($item) {
        case 'isJustifiable' :
            if (isJustifiableOfLvType($this->idEmployee, $this->idLeaveType)) {
                $result = '<div class="messageNotificationWarning" >';
                $result .= i18n('ThisLeaveMustBeJustified');
                $result .= '</div>';            
            } else {
                $result="";
            }
            break;
        //ajout eliott
        case 'startAMPM' :
            $result=$this->drawStartAMPM($readOnly);
            break;
        case 'endAMPM' :
            $result=$this->drawEndAMPM($readOnly);
            break;
        case 'nbDaysRemaining':
            $result=$this->drawNbDaysRemaining();
            break;
        //fin ajout
    }    
     return $result;
  }
  
  
  /**
   * to draw the spe startAMPM
   * @return string
   */ 
  public function drawStartAMPM($theReadOnly){
      if (!$theReadOnly) {  
        $readOnly = (strpos($this->getFieldAttributes("startAMPM"),"readonly")===false?"":"readonly");
    } else {
        $readOnly= "readonly";
    }    
    $idUser = getSessionUser()->id;
    //pour check les checkboxes selon les valeurs sur la bd, to check the checkboxes according to the values of startAMPM in the db
    $inputAM='<div style="display:inline-block;">';
    $inputPM='<div style="display:inline-block;">';
    
    if($this->idStatus!=NULL and $this->idStatus!=1){
        $inputAM.='<input data-dojo-type="dijit/form/CheckBox" id="startAM" name="startAM" readonly onchange="changesStartAM('.$idUser.','.$this->nbDays.','.$this->idStatus.','.$this->idLeaveType.')" ';
        $inputPM.='<input data-dojo-type="dijit/form/CheckBox" id="startPM" name="startPM" readonly onchange="changesStartPM('.$idUser.','.$this->nbDays.','.$this->idStatus.','.$this->idLeaveType.')" ';
    }else{
        $inputAM.='<input data-dojo-type="dijit/form/CheckBox" id="startAM" name="startAM" '.$readOnly.' onchange="changesStartAM('.$idUser.','.$this->nbDays.','.$this->idStatus.','.$this->idLeaveType.')" ';
        $inputPM.='<input data-dojo-type="dijit/form/CheckBox" id="startPM" name="startPM" '.$readOnly.' onchange="changesStartPM('.$idUser.','.$this->nbDays.','.$this->idStatus.','.$this->idLeaveType.')" ';
    }
    
    if($this->id==NULL){
        $inputAM.='checked>';
        $inputPM.='>';
    }else{
        $crit= array('id'=>$this->id);
        $lvResp = $this -> getSqlElementsFromCriteria ( $crit );
        if($lvResp[0]->startAMPM=='AM'){
            $inputAM.='checked>';
            $inputPM.='>';
        }else if($lvResp[0]->startAMPM=='PM'){
            $inputAM.='>';
            $inputPM.='checked>';
        }
    }
    $result  = '<tr class="detail generalRowClass startAMPMClass"><td class="label" style="text-align:right;width:145px">'
            . '<label for="startAMPM" class="generalColClass startAMPMClass" style=";">'.i18n("StartAMPM").'&nbsp;:&nbsp;</label></td>';
    $result .= '<td style="width:436px;">';
    $result .=  $inputAM;   
    $result .= '<label style="width: 23px;" for="startAM">AM</label></div>';
    $result .=    $inputPM ;
    $result .= '<label style="width: 23px;" for="startPM">PM</label></div>';
    $result .='</td></tr>';
    return $result;
  }
  
  /**
   * to draw the spe startAMPM
   * @return string
   */ 
  public function drawEndAMPM($theReadOnly){
    if (!$theReadOnly) {  
        $readOnly = (strpos($this->getFieldAttributes("startAMPM"),"readonly")===false?"":"readonly");
    } else {
        $readOnly= "readonly";
    }    
    $idUser = getSessionUser()->id;
//    $inputAM= '<div style="display:inline-block;"><input data-dojo-type="dijit/form/CheckBox" id="endAM" name="endAM" onchange="changesEndAM('.$idUser.')" ';
//    $inputPM= '<div style="display:inline-block;"><input data-dojo-type="dijit/form/CheckBox" id="endPM" name="endPM" onchange="changesEndPM('.$idUser.')" ';

    $inputAM= '<div style="display:inline-block;"> ';
    $inputPM= '<div style="display:inline-block;"> ';
    
    if($this->idStatus!=NULL and $this->idStatus!=1){
        $inputAM.= '<input data-dojo-type="dijit/form/CheckBox" id="endAM" name="endAM" readonly onchange="changesEndAM('.$idUser.','.$this->nbDays.','.$this->idStatus.','.$this->idLeaveType.')" ';
        $inputPM.= '<input data-dojo-type="dijit/form/CheckBox" id="endPM" name="endPM" readonly onchange="changesEndPM('.$idUser.','.$this->nbDays.','.$this->idStatus.','.$this->idLeaveType.')" ';
    }else{
        $inputAM.= '<input data-dojo-type="dijit/form/CheckBox" id="endAM" name="endAM" '.$readOnly.' onchange="changesEndAM('.$idUser.','.$this->nbDays.','.$this->idStatus.','.$this->idLeaveType.')" ';
        $inputPM.= '<input data-dojo-type="dijit/form/CheckBox" id="endPM" name="endPM" '.$readOnly.' onchange="changesEndPM('.$idUser.','.$this->nbDays.','.$this->idStatus.','.$this->idLeaveType.')" ';
    }
    if($this->id==NULL){
        $inputAM.= '>';
        $inputPM.= 'checked>';
    }else{
        $crit= array('id'=>$this->id);
        $lvResp = $this -> getSqlElementsFromCriteria ( $crit );
        if($lvResp[0]->endAMPM=='AM'){
            $inputAM.= 'checked>';
            $inputPM.= '>';
        }else if($lvResp[0]->endAMPM=='PM'){
            $inputAM.= '>';
            $inputPM.= 'checked>';
        }
    } 
    $result  = '<tr class="detail generalRowClass endAMPMClass"><td class="label" style="text-align:right;width:145px">'
            . '<label for="endAMPM" class="generalColClass endAMPMClass" style=";">'.i18n("EndAMPM").'&nbsp;:&nbsp;</label></td>';
    $result .= '<td style="width:436px;">';
    $result .=  $inputAM;
    $result .= '<label style="width: 23px;" for="endAM">AM</label></div>';
    $result .=  $inputPM;
    $result .= '<label style="width: 23px;" for="endPM">PM</label></div>';
    $result .=  '</td></tr>';  
    
    return $result;
  }
  
  /**
   * to draw the spe nbDaysRemaining
   * @return string
   */ 
  public function drawNbDaysRemaining(){
    $value=$this->getRemainingDays(0, $this->idEmployee,$this->idLeaveType);
    if ($value==null) {
        $value="";
    }
    
    $display = "display:inline-block;";        
      
    $result='<tr class="detail generalRowClass nbDaysRemainingClass"><td class="label" style="text-align:right;width:145px">'
          . '<label for="nbDaysRemaining" class="generalColClass nbDaysRemaining" style=";">'.i18n("nbDaysRemaining").'&nbsp;:&nbsp;</label></td><td style="width:436px;">';
    $result .=  '<input data-dojo-type="dijit/form/TextBox" id="idNbDaysRemaining" style="'.$display.' width: 4.25em;text-align: right;" readonly value='.$value.'></td></tr>';
    return $result;
  }
  
 /* ========================================================================================
  * MISCELANIOUS FUNCTIONS
    ======================================================================================== */
  
    /**
     * Get in array idProject, idActivity, idAssignment of this leave
     * @return Array[ idProject / idActivity / idAssignment ]
     */
    function getIdProjectIdActivityIdAssignmentOfThis() {
        $result = array();
        $result['idProject'] = Project::getLeaveProjectId();
        $lvType = new LeaveType($this->idLeaveType);
        $result['idActivity'] = $lvType->idActivity;
        $critRqAss=array(
            "idResource" => $this->idEmployee,
            "idProject" => $result['idProject'],
            "refType" => "Activity",
            "refId"=>$result['idActivity']
        );
        $result['idAssignment']=null;
        $ass = sqlElement::getSingleSqlElementFromCriteria("Assignment", $critRqAss);
        if (isset($ass->id)) {
            $result['idAssignment']=$ass->id;
        }
        return $result;
    }
  
    /**
     * Create work or planned work (depending of parameter) for this leave
     * @param string $workOrPlannedWork : 'Work' or 'PlannedWork'
     */
    private function createWorkOrPlannedWorkForLeave($workOrPlannedWork) {
        $pjLeaveId = Project::getLeaveProjectId();
        $lvType = new LeaveType($this->idLeaveType);
        $lvActId = $lvType->idActivity;
        $critRqAss=array(
            "idResource" => $this->idEmployee,
            "idProject" => $pjLeaveId,
            "refType" => "Activity",
            "refId"=>$lvActId
        );
        $ass = new Assignment();
        $assRq = $ass->getSqlElementsFromCriteria($critRqAss);
        
        $startDateTime = new Datetime($this->startDate);
        $endDateTime = new Datetime($this->endDate);
        $itDateTime = $startDateTime;
        
        while($itDateTime<=$endDateTime){
            $workOrPWork=new $workOrPlannedWork();
            if($itDateTime==$startDateTime and $this->startAMPM=="PM"){
                $workOrPWork->work=0.5;
            }else if($itDateTime==$endDateTime and $this->endAMPM=="AM"){
                $workOrPWork->work=0.5;
            }else{
                $workOrPWork->work=1.0;
            }
            $workOrPWork->idResource=$this->idEmployee;
            $workOrPWork->idProject=$pjLeaveId;
            $workOrPWork->refType="Activity";
            $workOrPWork->refId=$lvActId;
            $workOrPWork->idAssignment=$assRq[0]->id;
            $workOrPWork->setDates($itDateTime->format('Y-m-d'));
            $workOrPWork->idLeave = $this->id;
            $resultS=$workOrPWork->simpleSave();
            if(strpos($resultS, "OK")===false){
                return $resultS;   
            }
            $itDateTime->add(new DateInterval('P1D'));
        }        
    }
      
    /**
    * return a list of the leaveEarned of this leave
    * @param int idLeaveType
    * @param int idEmployee
    * @param boolean $reverse if true, Order by startDate DESC, else by startDate ASC
    * @return Array[EmployeeLeaveEarned] 
    */
    protected function getEmpLeaveEarnedListOfThisLeave($idLeaveType,$idEmployee,$reverse=false) {
      $clauseWhere="idLeaveType=".$idLeaveType." AND idEmployee=".$idEmployee." AND idle=0";
      if($reverse){
          $clauseOrderBy="startDate DESC";
      }else{
          $clauseOrderBy="startDate ASC";
      }
      $empLE=new EmployeeLeaveEarned();
      $empLEList = $empLE->getSqlElementsFromCriteria(null, false,$clauseWhere,$clauseOrderBy);
      return $empLEList;
    }

    /**
     * calculate the attribute nbDays of this leave
     */
    public function calculateNbDays($fromCalendarDefinition=false){
        $nbEffectiveLeaveDays=0;
        if ($this->idEmployee == getSessionUser()->id) { 
            $idCalDef = getSessionUser()->idCalendarDefinition;
        } else {
            $empl = new Resource($this->idEmployee);
            $idCalDef = $empl->idCalendarDefinition;
        }
        $cal=new Calendar();
        $cal->setDates(date('Y').'-01-01');
        $arrayOffDays=explode("#", $cal->getOffDayList($idCalDef) );
        $arrayExceptionnalWorkDays=explode("#", $cal->getWorkDayList($idCalDef) );
        $arrayOffDaysOfTheWeek = $this -> arrayOfOffDaysOfTheWeek($fromCalendarDefinition);
        $startDay=new DateTime($this->startDate);
        $currentDay=new DateTime($this->startDate);
        $lastDay=new DateTime($this->endDate);
        
        while($currentDay <= $lastDay){
            if( ($arrayOffDaysOfTheWeek[ date_format($currentDay,'N') ] === 0 && in_array($currentDay->format('Ymd'), $arrayOffDays)==FALSE) || in_array($currentDay->format('Ymd'), $arrayExceptionnalWorkDays)==TRUE){
                if($this->startAMPM=="PM" && $this->endAMPM=="AM" && $startDay==$lastDay){
                    //pass
                }else if($this->startAMPM=="PM" && $startDay==$currentDay){
                    $nbEffectiveLeaveDays+=0.5;
                }else if($this->endAMPM=="AM" && $lastDay==$currentDay){
                    $nbEffectiveLeaveDays+=0.5;
                }else{
                    $nbEffectiveLeaveDays+=1;
                }
            }
            
            $currentDay->add(new DateInterval('P1D'));
        }
        $this->nbDays=$nbEffectiveLeaveDays;
 }
  
/**
 * ajout eliott: function which return an array containing the days of the week which are an offDay in general, 
 * the first day of the string is sunday, if there is a 1, this day is an offDay, used in save() to calculate nbDays
 * @return array
 */
  public function arrayOfOffDaysOfTheWeek($directFromDatabase=false){
    $isOffDay=array();
    if ($directFromDatabase) {
        $in = "('OpenDayMonday', 'OpenDayTuesday', 'OpenDayWednesday', 'OpenDayThursday', 'OpenDayFriday', 'OpenDaySaturday', 'OpenDaySunday')";
        $p=new Parameter();
        $crit=" (idUser IS NULL AND idProject IS NULL AND parameterCode IN $in)";
        $lst=$p->getSqlElementsFromCriteria(null, false, $crit);
        foreach ($lst as $param) {
            if ($param->parameterCode=='OpenDayMonday') {
                $isOffDay['1'] = ($param->parameterValue=="offDays"?1:0);
            } elseif ($param->parameterCode=='OpenDayTuesday') {
                $isOffDay['2'] = ($param->parameterValue=="offDays"?1:0);                
            } elseif ($param->parameterCode=='OpenDayWednesday') {
                $isOffDay['3'] = ($param->parameterValue=="offDays"?1:0);                
            } elseif ($param->parameterCode=='OpenDayThursday') {
                $isOffDay['4'] = ($param->parameterValue=="offDays"?1:0);                
            } elseif ($param->parameterCode=='OpenDayFriday') {
                $isOffDay['5'] = ($param->parameterValue=="offDays"?1:0);                
            } elseif ($param->parameterCode=='OpenDaySaturday') {
                $isOffDay['6'] = ($param->parameterValue=="offDays"?1:0);                
            } elseif ($param->parameterCode=='OpenDaySunday') {
                $isOffDay['7'] = ($param->parameterValue=="offDays"?1:0);                
            }
        }
    } else {
        $isOffDay['1']=(Parameter::getGlobalParameter('OpenDayMonday')=="offDays"?1:0);
        $isOffDay['2']=(Parameter::getGlobalParameter('OpenDayTuesday')=="offDays"?1:0);
        $isOffDay['3']=(Parameter::getGlobalParameter('OpenDayWednesday')=="offDays"?1:0);
        $isOffDay['4']=(Parameter::getGlobalParameter('OpenDayThursday')=="offDays"?1:0);
        $isOffDay['5']=(Parameter::getGlobalParameter('OpenDayFriday')=="offDays"?1:0);
        $isOffDay['6']=(Parameter::getGlobalParameter('OpenDaySaturday')=="offDays"?1:0);
        $isOffDay['7']=(Parameter::getGlobalParameter('OpenDaySunday')=="offDays"?1:0);
    }
    return $isOffDay;
  }
  
  
  /**
   * return an array of the query to know the quantity of the leavetype of the leave
   * 
   * @return array  an array containing an object employeeleaveearned if idLeaveType is set, else an array containing the string "KO"
   */
  public function getQuantityStartDateEndDateOfLeaveType(){
      if($this->idLeaveType==NULL){
          return array("KO");
      }
      //if it's a new leave,
      if($this->id==NULL){
          $crit= array("idLeaveType" => $this->idLeaveType, "idEmployee" => getSessionUser () -> id);
      }else{//if the leave was already created
        $crit= array("idLeaveType" => $this->idLeaveType, "idEmployee" => $this->idEmployee);
      }
      $empLE= new EmployeeLeaveEarned();
      $critQuantityStartEndDateList= $empLE->getSqlElementsFromCriteria($crit);
      return $critQuantityStartEndDateList;      
  }
  
  
  /**
   * return the number of days remaining for a leave of a given leavetype
   * @param type $idle
   * @param type $idEmployee
   * @param type $idLeaveType
   * @return int/string: if the leavetype has a quantity then the number returned is the number of days remaining, else it's ""
   */
  public function getRemainingDays($idle=0,$idEmployee=null,$idLeaveType=null){        
      if ($idEmployee==null or $idLeaveType==null) {return "";}
      
      $emp = new Employee($idEmployee);
      $leftList = $emp->getLeftLeavesByLeaveType($idLeaveType);
      if ($leftList) {
          return $leftList[$idLeaveType];
      } else {
          return "";
      }
//      $lvE=new EmployeeLeaveEarned();
//      $date = (new DateTime())->format("Y-m-d");
//      $crit = "idle = $idle AND idEmployee = $idEmployee AND idLeaveType = $idLeaveType AND (startDate<='$date' OR startDate IS NULL)";
//      $lvEarnedList = $lvE->getSqlElementsFromCriteria(null,false,$crit);
//      if($lvEarnedList){
//          if($lvEarnedList[0]->quantity !== null){
//              $leftQTotal=0;
//              foreach($lvEarnedList as $lvEarned){
//                $contract = null;
//                if ($lvEarned->startDate!=null) {
//                    $actualContractualValue = getActualLeaveContractualValues($idEmployee,$lvEarned->idLeaveType,$contract);
//                    $validityDuration = $actualContractualValue->validityDuration;
//                    $theValidityDate = (new DateTime($lvE->startDate))->add(new DateInterval("P".$validityDuration."M"))->format("Y-m-d");
//                    if ($theValidityDate>=$date and $lvEarned->leftQuantity!=null) {
//                        $leftQTotal += $lvEarned->leftQuantity;
//                    }
//                }  
//              }
//              return $leftQTotal;
//          }else{
//              return "";
//          }
//    }else{
//      return "";
//    }
  }
  
  
  /**
   * return a string containing a date (format Y-m-d) which is the first open day that doesn't overlap with the dates of already existing leaves 
   * to prefill the attributes startDate and endDate at the creation of a new leave
   * 
   * @return string a string containing the first open date available at the creation of a leave (id=null) 
   */
  public function getFirstOpenDateAtCreation(){
    $today=(new DateTime())->format('Ymd'); 
    
    //an array containing all the exceptionnal offdays of the current year
    $arrayOffDays=explode("#", Calendar::getOffDayList(getSessionUser()->idCalendarDefinition) );
    
    //an array containing the seven of the week and if each one is an openDay or OffDay
    $arrayOffDaysOfTheWeek = $this -> arrayOfOffDaysOfTheWeek();

    //a request to get all the leaves the employee asking for a new leave has already taken with a date superior than the date the leave was created
    $clauseWhereSelectLvs = "idEmployee= ".$this->idEmployee." AND (startDate >= ".$today." OR endDate >= ".$today.") AND idStatus <> 9";
    $clauseOrderByAscDates = "startDate, endDate ASC";
    $lvList=$this->getSqlElementsFromCriteria(null, false, $clauseWhereSelectLvs, $clauseOrderByAscDates);
    
    //the iterator for the new date
    $current=(new DateTime($today));
    
    //the condition for the while
    $cond=false;
    //two iterators to iterate in $lvList
    $iterator=0;
    $oldIt=0;
    $len = count($lvList);


    //while an appropriate date was not found
    while(! $cond){
        //if there are still leaves in $lvList
        
        if($iterator< $len){
          //if $current is between the dates of the leave then it becomes the endDate of the leave +1 d
          $lvStart = new DateTime($lvList[$iterator]->startDate);
          $lvEnd = new DateTime($lvList[$iterator]->endDate);
        
          if( $lvStart <= $current and $lvEnd >= $current){

              if($iterator<$len-1){
                  $lvStartNext = new DateTime($lvList[$iterator+1]->startDate);
                  $lvEndNext = new DateTime($lvList[$iterator+1]->endDate);
                  //to increment the iterator of one if there is two leaves on the same days like in this example : (15/06/18 to 18/06/18 endAMPM : AM and 18/06/18 to 18/06/18 startAMPM = endAMPM = PM)
                  if(($lvStart <= $current and $lvEnd >= $current && $lvEnd==$lvStartNext && $lvStartNext==$lvEndNext)){
                      $iterator++;
                  }
                    $current=(new DateTime($lvList[$iterator]->endDate))->add(new DateInterval('P1D'));
              }else{
                  $current=(new DateTime($lvList[$iterator]->endDate))->add(new DateInterval('P1D'));
              }
            $oldIt=$iterator;
            $iterator++;
            }
        }
        //if current was modified in this iteration then iterate once more to be able to test 
        //if there is two consecutives leaves in $lvList 
        if($oldIt!=$iterator){
            $oldIt=$iterator;
        }else{//else if $current was not modified in this iteration yet then            
            //if current is not an offday (week end) and not an exceptionnal off day then current is the first appropriate open day and the while can end
            if( $arrayOffDaysOfTheWeek[ date_format($current,'N') ] === 0 and in_array($current->format('Ymd'), $arrayOffDays)===FALSE){
                $cond = true;
            }else{//else $current +1 d
                $current=$current->add(new DateInterval('P1D'));
            }
        }
    }
    
    return $current->format("Y-m-d");
  }
  
  /**
   * to update the leftQuantity attributes of the LeaveEarned linked to this leave (called in save and delete of LeaveMain)
   * @param int $idLeaveType
   * @param int $idEmployee
   * @param int $nbDays
   * @param int $oldDays (needed if $update == true)
   * @param bool $create
   * @param bool $update
   * @param bool $delete
   * @return string 
   */
  function updateLeftQOfLeaveEarned($idLeaveType,$idEmployee,$nbDays,$oldDays=null,$create=false,$update=false,$delete=false){
      //if $oldDays isn't set and $update==true, stop
      $returnErrorPrefix="<b>Invalid controls.</b><br/>";
      $returnErrorHiddenPart = '<input type="hidden" id="lastSaveId" value="' . htmlEncode ( $this->id ) . '" />';
      $returnErrorHiddenPart .= '<input type="hidden" id="lastOperation" value="save" />';
      $returnErrorHiddenPart .= '<input type="hidden" id="lastOperationStatus" value="INVALID" />';
      
      $returnErrorCantTakeMoreLeaves = $returnErrorPrefix . '<br/><b>' . i18n('ErrorCantTakeMoreLeaves') . '</b><br/>' . $returnErrorHiddenPart;
      
      if(($update==true) && $oldDays==null){
          return $returnErrorPrefix . '<br/><b>' . i18n('errorMissingOldDaysInUpdateLeftInLeaveMain') . '</b><br/>' . $returnErrorHiddenPart;
    //error
      }
      
      //to test if there is only one mode set to true
      $testModeArray=array(
                        "create"=>"$create",
                        "update"=>"$update",
                        "delete"=>"$delete"
                      );
      if( array_count_values($testModeArray)["1"]!=1 ){
          return $returnErrorPrefix . '<br/><b>' . i18n('errorModeUpdateLeftInLeaveMain') . '</b><br/>' . $returnErrorHiddenPart;
      }
      
      $empLEList = $this->getEmpLeaveEarnedListOfThisLeave($idLeaveType,$idEmployee);
      
      if(! $empLEList){
          return $returnErrorPrefix . '<br/><b>' . i18n('ErrorThisEmployeeDoesntHaveLeaveEarned') . '</b><br/>' . $returnErrorHiddenPart;
      }
      //if there is no quantity, the employee can ask as many leaves as he wants, 
      //so there is no need to modify leftQuantity 
      if($empLEList[0]->quantity==null){
          return "";
      }
      
//      $isAnticipated=isAnticipatedOfLvType($this->idEmployee,$this->idLeaveType);
      $nbAllowedAnticipated = getNbAnticipatedAllowedFormEmployeeAndLeaveType($this->idEmployee, $this->idLeaveType);
      
      //if there is only one limited leaveEarned
      if(count($empLEList)==1){
          if ($create==true){
              $nDaysToSubstract=$empLEList[0]->leftQuantity - $nbDays;
              //if there is not enough quantity
//              if ($nDaysToSubstract < 0 && !$isAnticipated)){
              if ($nDaysToSubstract < 0 && $nbAllowedAnticipated < abs($nDaysToSubstract)){
                  return $returnErrorCantTakeMoreLeaves;
              }
//              if ( ($nDaysToSubstract >= 0) || ($isAnticipated && $nDaysToSubstract < 0) ){
              if ( ($nDaysToSubstract >= 0) || ($nbAllowedAnticipated >= abs($nDaysToSubstract)) ){
                  $empLEList[0]->leftQuantity=$nDaysToSubstract;
                  $resultS=$empLEList[0]->simpleSave();
                  if(strpos($resultS, "OK")===false && strpos($resultS, "NO_CHANGE")===false){
                    return $resultS;   
                  }
              }
              
          } else if($update==true){
              //$diff=$empLEList[0]->leftQuantity - ($nbDays - $oldDays);
              $diffNbDays = ($nbDays - $oldDays);
              if($diffNbDays==0){
                  
              } else if($diffNbDays<0){
                  $diff = min(-$diffNbDays, $empLEList[0]->quantity - $empLEList[0]->leftQuantity);
                  $empLEList[0]->leftQuantity=$empLEList[0]->leftQuantity+$diff;
                  $resultS=$empLEList[0]->simpleSave();
                  if(strpos($resultS, "OK")===false && strpos($resultS, "NO_CHANGE")===false){
                    return $resultS;   
                  }
              } else if($diffNbDays>0){
                  $diff = $empLEList[0]->leftQuantity - $diffNbDays;
//                  if(!$isAnticipated && $diff < 0){
                  if($nbAllowedAnticipated < abs($diff) && $diff < 0){
                      return $returnErrorCantTakeMoreLeaves;
                  }
//                  if( ($diff >= 0) || ($isAnticipated && $diff < 0) ){
                  if( ($diff >= 0) || ($nbAllowedAnticipated >= abs($diff)) ){
                      $empLEList[0]->leftQuantity=$diff;
                      $resultS=$empLEList[0]->simpleSave();
                      if(strpos($resultS, "OK")===false && strpos($resultS, "NO_CHANGE")===false){
                        return $resultS;   
                      }
                  }
              }
              
              
          } else if($delete==true){
              $empLEList[0]->leftQuantity=$empLEList[0]->leftQuantity + min($nbDays,$empLEList[0]->quantity-$empLEList[0]->leftQuantity);
              $resultS=$empLEList[0]->simpleSave();
              if(strpos($resultS, "OK")===false && strpos($resultS, "NO_CHANGE")===false){
                return $resultS;
              }
          }
      
    //if $empLEList is composed of more than one leaveEarned
      } else {
          $empLEListLength = count($empLEList);
          $nbDaysIterator=$nbDays;
          $totalQuantityLeft=0;
          foreach($empLEList as $lvEarned){
              $totalQuantityLeft+=$lvEarned->leftQuantity;
          } 
          
          if($create==true){
              $empLEList = $this->getEmpLeaveEarnedListOfThisLeave($idLeaveType,$idEmployee);              
//              if($totalQuantityLeft < $nbDays && !$isAnticipated){
              if($totalQuantityLeft < $nbDays && ($nbAllowedAnticipated < abs($nbDays-$totalQuantityLeft))){
                  return $returnErrorCantTakeMoreLeaves;
              }
              for($i=0; $i < $empLEListLength-1; $i++){
                  if($nbDaysIterator==0){
                      break;
                  }else{
                      $decrement=min($empLEList[$i]->leftQuantity,$nbDaysIterator);
                      $empLEList[$i]->leftQuantity = $empLEList[$i]->leftQuantity - $decrement;
                      $nbDaysIterator=$nbDaysIterator-$decrement;
                      $resultS=$empLEList[$i]->simpleSave();
                      if(strpos($resultS, "OK")===false && strpos($resultS, "NO_CHANGE")===false){
                        return $resultS;
                      }
                      //error
                  }
              }
              if($nbDaysIterator>0){
                $decrement=$empLEList[$empLEListLength-1]->leftQuantity - $nbDaysIterator;
                //error decrement < 0 and not isAnticipated ?
                $empLEList[$empLEListLength-1]->leftQuantity=$empLEList[$empLEListLength-1]->leftQuantity-$nbDaysIterator;
                $resultS=$empLEList[$empLEListLength-1]->simpleSave();
                if(strpos($resultS, "OK")===false && strpos($resultS, "NO_CHANGE")===false){
                  return $resultS;   
                }
                
              }
          }
          
          if($update==true){
              $diff = $nbDays - $oldDays;
              if($diff==0){ 
                  //pass
              } else if($diff>0){
//                  if($totalQuantityLeft < $diff && !$isAnticipated){
                  if($totalQuantityLeft < $diff && ($nbAllowedAnticipated < abs($diff-$totalQuantityLeft))){
                    return $returnErrorCantTakeMoreLeaves;
                  }
                  $empLEList = $this->getEmpLeaveEarnedListOfThisLeave($idLeaveType,$idEmployee);
                  for($i=0; $i < $empLEListLength-1; $i++){
                    if($diff==0){
                        break;
                    }else{
                        $decrement=min($diff,$empLEList[$i]->leftQuantity);
                        $empLEList[$i]->leftQuantity=$empLEList[$i]->leftQuantity-$decrement;
                        $diff=$diff-$decrement;
                        $resultS=$empLEList[$i]->simpleSave();
                        if(strpos($resultS, "OK")===false && strpos($resultS, "NO_CHANGE")===false){
                          return $resultS;
                        }
                        
                    }
                }
                if($diff!=0){
                    $empLEList[$empLEListLength-1]->leftQuantity = $empLEList[$empLEListLength-1]->leftQuantity - $diff;
                    $resultS=$empLEList[$empLEListLength-1]->simpleSave();
                    if(strpos($resultS, "OK")===false && strpos($resultS, "NO_CHANGE")===false){
                      return $resultS;   
                    }
                    
                }
              }else if($diff<0){
                  $empLEList = $this->getEmpLeaveEarnedListOfThisLeave($idLeaveType,$idEmployee,true);
                  for($i=0; $i < $empLEListLength; $i++){
                    if($diff==0){
                        break;
                    }else{
                        $increment = min(-$diff,$empLEList[$i]->quantity-$empLEList[$i]->leftQuantity);
                        $empLEList[$i]->leftQuantity=$empLEList[$i]->leftQuantity + $increment;
                        $diff=$diff + $increment;
                       $resultS=$empLEList[$i]->simpleSave();
                       if(strpos($resultS, "OK")===false && strpos($resultS, "NO_CHANGE")===false){
                         return $resultS;   
                       }
                    }
                  } 
              }
              
          }
          
          if($delete==true){
              $nbDaysToAdd = $nbDays;
              $empLEList = $this->getEmpLeaveEarnedListOfThisLeave($idLeaveType,$idEmployee,true);
              $empLEListLength=count($empLEList);
              $nbDaysToAdd=$nbDays;
              for($i=0; $i < $empLEListLength; $i++){
                  if($nbDaysToAdd==0){
                      break;
                  }else{
                      $increment=min($empLEList[$i]->quantity - $empLEList[$i]->leftQuantity, $nbDaysToAdd);
                      $empLEList[$i]->leftQuantity = $empLEList[$i]->leftQuantity + $increment;
                      $nbDaysToAdd = $nbDaysToAdd - $increment;
                      $resultS=$empLEList[$i]->simpleSave();
                      if(strpos($resultS, "OK")===false && strpos($resultS, "NO_CHANGE")===false){
                        return $resultS;
                      }
                  }
              }
          }
          
      }
      return "";
  }  
//fin ajout

  /**
   * Determine if information on leave is to send.
   * @param string $action : Action doing on the leave. Avalaible values are : OnCreate, OnUpdate, OnDelete, OnTreatment ie change of status)
   * @param string $typeOfSend : Way of sending. Avalaible values are : notification, email, alert
   * @param string $typeOfReceiver : Type of receiver. Avalaible values are : A = LeaveAdministrator - M = Manager of employee - E = Employee
   * @return boolean : True if information on leave is to send by the typeOfSend, for the action and the type of receiver
   */
  private function isInfoOnLeaveToSend($action=null, $typeOfSend=null, $typeOfReceiver=null) {
      $theLeaveType = new LeaveType($this->idLeaveType);
      if ($action==null or $typeOfReceiver==null or $typeOfSend==null) { return false; }
      $theField = $typeOfSend.$action;
      if (strpos($theLeaveType->$theField, $typeOfReceiver)===false) {return false; }
      return true;
  }
  
  /**
   * Send information on leave by notification, alert, email in function of the type of leave
   * @param Leave $old : Old values of the leave
   * @param boolean $delete : True if deleting of leave
   * @return nothing
   */
  private function sendLeaveInfo($old,$delete=false) {
    // RECEIVERS AND REQUESTER
    $receivers = array();
    $requester = null;
    // Get employee's leave
    $employee = new Employee($this->idEmployee);
    // Get manager of this employee's leave
    $manager = $employee->getActivManager();
    // Get leaves admin
    $leavesAdmin = getLeavesAdmin();
    // Determine the receiver
    if ($this->idUser == $this->idEmployee) { // Creator = Employee
        // receiver = manager
        $manager->__Ame = "M";
        array_push($receivers, $manager);
        // requester = Employee
        $requester = $employee;
        $who = i18n("employee");
    } elseif ($this->idUser == $manager->id) { // Creator = Manager
        // receiver = Employee
        $employee->__Ame = "E";
        array_push($receivers, $employee);
        // requester = Manager
        $requester = $manager;            
        $who = i18n("manager");
    } elseif ($this->idUser == $leavesAdmin->id) { // Creator = Leaves Admin (that is'nt the actual employee's manager)
        // receivers = Employee - Manager
        $manager->__Ame = "M";
        array_push($receivers, $manager);
        $employee->__Ame = "E";
        array_push($receivers, $employee);
        // requester = Leaves admin
        $requester = $leavesAdmin;            
        $who = i18n("leavesAdmin");
    } else { // Something goes wrong : Leave requester is employee or it's manager or the leave admin
        // receivers = Employee - Manager - Leaves admin
        $manager->__Ame = "M";
        array_push($receivers, $manager);
        $employee->__Ame = "E";
        array_push($receivers, $employee);
        $leavesAdmin->__Ame = "M";
        array_push($receivers, $leavesAdmin);            
        // requester = the connected user
        $requester = getSessionUser();     
        $who = i18n("User");
    }
    
    // Don't send Notification or email or alert to him self
    foreach($receivers as $key => $receiver) {
        if ($receiver->id == $this->idUser) {
            unset($receivers[$key]);
        }
    }
    
    if (empty($receivers)) {
        return;
    }

    $newLeaveType = new LeaveType($this->idLeaveType);

    // CAUSE
    $cause = "";
    $action = "";
    // Determine cause of save
    if ($delete) {
        $action = "OnDelete";
        $cause = i18n("Deleted");
    } elseif ($old->id==null) { // New Leave
        $action = "OnCreate";
        $cause = i18n("Requested");
    } else {
        if (($old->idStatus==1 or $old->idStatus==null) and $this->idStatus==9) {
            $action = "OnTreatment";
            $cause = i18n("Rejected");
        }
        if (($old->idStatus==1 or $old->idStatus==null) and $this->idStatus==12) {
            $action = "OnTreatment";
            $cause .= ($cause==""?"":" ".i18n("AND")." ").i18n("Validated");
        } 
        if ($old->idStatus==9 and $this->idStatus==1) {
            $action = "OnTreatment";
            $cause .= ($cause==""?"":" ".i18n("AND")." ").i18n("ReopenedAfterRejected");
        }
        if ($old->idStatus==12 and $this->idStatus==1) {
            $action = "OnTreatment";
            $cause .= ($cause==""?"":" ".i18n("AND")." ").i18n("ReopenedAfterValidated");
        }
        if ($old->idLeaveType != $this->idLeaveType and $old->idLeaveType!=null) {
            $oldLeaveType = new LeaveType($old->idLeaveType);
            $cause .= ($cause==""?"":" ".i18n("AND")." ").i18n("changeLeaveType")." ".i18n("from")." ".$oldLeaveType->name." ".i18n("to")." ".$newLeaveType->name;
        }
        if ($old->startDate != $this->startDate or
                   $old->startAMPM != $this->startAMPM or
                   $old->endDate != $this->endDate or
                   $old->endAMPM != $this->endAMPM
                 ) {
            $action = "OnUpdate";
            $cause .= ($cause==""?"":" ".i18n("AND")." ").i18n("changeLeaveDate")." ";
            $cause .= i18n("old")." ".i18n("startDate"). " = ". $old->startDate. " ".$old->startAMPM." ";
            $cause .= i18n("endDate"). " = ". $old->endDate. " ".$old->endAMPM." ";
            $cause .= i18n("new")." ".i18n("startDate"). " = ". $this->startDate. " ".$this->startAMPM." ";
            $cause .= i18n("endDate"). " = ". $this->endDate. " ".$this->endAMPM;
        }
        if ($old->idEmployee != $this->idEmployee) {
            $action = "OnUpdate";
            $cause .= ($cause==""?"":" ".i18n("AND")." ").i18n("changeLeaveEmployee");
        }
    }

    if ($cause=="") {
        return;
    }
    
    if (isNotificationSystemActiv()) {
        // NOTIFICATIONS
        $menu = SqlElement::getFirstSqlElementFromCriteria("Menu", array("name" => "menuLeave"));
        if (!isset($menu->id)) {
            $idMenu = null;
        } else {
            $idMenu = $menu->id;
        }
        $notifType = SqlElement::getFirstSqlElementFromCriteria("Type", array("name" => "INFO", "scope" => "Notification"));
        if (!isset($notifType->id)) {
            $notifType = SqlElement::getFirstSqlElementFromCriteria("Type", array("scope" => "Notification"));
        }
        if (isset($notifType->id)) {
            $idNotifType = $notifType->id;
        } else {
            $idNotifType = null;
        }
        $notifiable = SqlElement::getFirstSqlElementFromCriteria("Notifiable", array("notifiableItem" => "Leave"));
        if (isset($notifiable->id)) {
            $idNotifiable = $notifiable->id;
        } else {
            $idNotifiable = null;
        }
        // Prepare notification values
        $notif = new Notification();
        $notif->idResource = $this->idUser;
        $notif->idNotifiable = $idNotifiable;
        $notif->notifiedObjectId = $this->id;
        $notif->idNotificationDefinition=null;
        $notif->idMenu=$idMenu;
        $notifDate = new DateTime();
        $notif->notificationDate = $notifDate->format("Y-m-d");
        $notif->notificationTime = $notifDate->format("H:i:s");
        $notif->idNotificationType = $idNotifType;
        if ($delete or $old->id==null) {
            $notif->name = i18n("leave")." #".$this->id." ".i18n("is")." ".$cause." - ".$newLeaveType->name;
            $content = i18n("startDate"). " = ". $this->startDate. " ".$this->startAMPM." ";
            $content .= i18n("endDate"). " = ". $this->endDate. " ".$this->endAMPM;
            $notif->content = $newLeaveType->name." ".$content." ".$this->comment;
        } else {
            $notif->name = $newLeaveType->name." #".$this->id." ".i18n("is")." ".i18n("changed")." ".i18n("by")." ".$requester->userName;
            $notif->content = $cause;
        }
        $notif->title = i18n("informations")." ".i18n("FOR")." ".i18n("leave")." #".$this->id;
        $notif->sendEmail = 0;
        $notif->idle=0;
    } 
    
    // For each receivers
    foreach($receivers as $key => $receiver) {
        $user = new User($receiver->id);
        // NOTIFICATIONS
        if ($this->isInfoOnLeaveToSend($action,'notification',$receiver->__Ame)==true) {
            // Must be set before access right
            $readAllowed=false;
            if (isNotificationSystemActiv()) {
                $notif->idUser = $receiver->id;
                // Get access to menu Notification for receiver
                $readAllowed = (securityGetAccessRightYesNo("menuNotification", "read", $notif, $user)=="YES");
            }
            // Notification allowed and Notification system activ
            if (($readAllowed) and isNotificationSystemActiv()) {
                // Send Notification
                $notif->id=null;
                $notif->simpleSave();
            }
        }
        
        // ALERT
        if ($this->isInfoOnLeaveToSend($action,'alert',$receiver->__Ame)==true) {
            // Must be set before access right
            $alert=new Alert();
            $alert->idUser=$receiver->id;
            // Get access to menu Alert
            $readAllowed = (securityGetAccessRightYesNo("menuAlert", "read", $alert, $user)=="YES");
            // Alert allowed
            if ($readAllowed) {
                // Emit alert
                if ($delete or $old->id==null) {
                    $title = i18n("leave")." ".i18n("is")." ".$cause." - ".$newLeaveType->name. " ". i18n("FOR")." ".$requester->name;
                    $message = i18n("by")." ".$user->name." ".i18n("comment")." : ".$this->comment;
                } else {
                    $title = i18n("leave")." ".i18n("is")." ".i18n("changed")." ".i18n("FOR")." ".$requester->name;
                    $message = $cause;
                }
                $theDate = new DateTime();
                
                $alert=new Alert();
                $alert->idUser=$receiver->id;
                $alert->alertType=htmlspecialchars("INFO",ENT_QUOTES,'UTF-8');
                $alert->alertInitialDateTime=$theDate->format("Y-m-d H:i:s");
                $alert->alertDateTime=$theDate->format("Y-m-d H:i:s");
                $alert->title=htmlspecialchars($title,ENT_QUOTES,'UTF-8');
                $alert->message=htmlspecialchars($message,ENT_QUOTES,'UTF-8');
                $alert->simpleSave();
            }
        }

        // EMAIL
        if ($this->isInfoOnLeaveToSend($action,'email',$receiver->__Ame)==true) {
            // If receiver as email => send email
            if ($receiver->email!=null) {
                if ($delete or $old->id==null) {
                    $subject = i18n("leave")." ".i18n("is")." ".$cause." - ".$newLeaveType->name." ".i18n("FOR")." ".$requester->name;
                    $messageBody = i18n("by")." ".$user->name." ".i18n("comment")." : ".$this->comment;
                } else {
                    $subject = i18n("leave")." ".i18n("is")." ".i18n("changed")." ".i18n("FOR")." ".$requester->name;
                    $messageBody = $cause;
                }
                sendMail($receiver->email, $subject, $messageBody);
            }
        }
    }    
  }  
}
?>
